import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.Color;
import java.io.Serializable;

/**
 * Die Klasse Moebel definiert die gemeinsamen Eigenschaften aller
 * Moebelklassen.
 * 
 * @author Claus Albowski
 * @version 2.2 (aug 07)
 */
public abstract class Furniture implements Serializable {

    private static final long serialVersionUID = 1L; // default serialID
    protected int xPosition;
    protected int yPosition;
    protected int orientation;
    protected Color color;
    protected Color currentColor; // this is copied from color once created
    protected Color defaultColor; // this is copied from color once created
    protected boolean isVisible;
    protected int width;
    protected int depth;
    protected int defaultWidth;
    protected int defaultDepth;

    // Kein Konstruktor !!!!

    /**
     * Diese Methode ist der Kern der Abstraktion.
     */
    protected abstract Shape getCurrentFigure();

    /**
     * Mache dieses Objekt sichtbar. Wenn es bereits sichtbar ist, tue nichts.
     */
    public void show() {
        if (!isVisible) {
            isVisible = true;
            draw();
        }
    }

    /**
     * transformiert das aktuelle Shape.
     */
    protected Shape transform(Shape shape) {
        AffineTransform t = new AffineTransform();
        t.translate(xPosition, yPosition);
        Rectangle2D umriss = shape.getBounds2D();
        t.rotate(Math.toRadians(orientation), umriss.getX() + umriss.getWidth() / 2,
                umriss.getY() + umriss.getHeight() / 2);
        return t.createTransformedShape(shape);
    }

    /**
     * Mache dieses Objekt unsichtbar. Wenn es bereits unsichtbar ist, tue nichts.
     */
    public void hide() {
        delete(); // "tue nichts" wird in delete() abgefangen.
        isVisible = false;
    }

    /**
     * Drehe auf den angegebenen Winkel
     */
    public void rotateTo(int newAngle) {
        delete();
        orientation = newAngle;
        draw();
    }

    public void resetCanvas() {
        delete();
        draw();
    }

    public void rotateAdd(int angleDifference) {
        delete();
        orientation += angleDifference;
        draw();
    }

    public void resetRotation() {
        delete();
        orientation = 0;
        draw();
    }

    public void changeObjectSize(int scrollDirection) { // scrollDirection is 1 or -1 depending of scrolled direction
        delete();
        if (scrollDirection == 1) {
            width += width / 15;
            depth += depth / 15;
        } else if (scrollDirection == -1 && width > 30 && depth > 30) { // depth or width shall not be under 30 to set a
                                                                        // minimum size
            width -= width / 15;
            depth -= depth / 15;
        }
        draw();
    }

    public void resetObjectSize() {
        delete();
        width = defaultWidth;
        depth = defaultDepth;
        draw();
    }

    /**
     * Bewege dieses Objekt horizontal um 'entfernung' Bildschirmpunkte.
     */
    public void moveHorizontal(int distance) {
        delete();
        xPosition += distance;
        draw();
    }

    /**
     * Bewege dieses objekt vertikal um 'entfernung' Bildschirmpunkte.
     */
    public void moveVertical(int distance) {
        delete();
        yPosition += distance;
        draw();
    }

    /**
     * Sets new x and y positions.
     */
    public void setPos(int xPos, int yPos) {
        delete();
        xPosition = xPos;
        yPosition = yPos;
        draw();
    }

    /**
     * Gets x position.
     */
    public int getXPos() {
        return xPosition;
    }

    /**
     * Gets y position.
     */
    public int getYPos() {
        return yPosition;
    }

    /**
     * Aendere die Farbe dieses Objektes in 'neueFarbe'. Gueltige Angaben sind
     * "rot", "gelb", "blau", "gruen", "lila" und "schwarz".
     */
    public void changeCurrentColor(Color newColor) {
        delete();
        currentColor = newColor;
        draw();
    }

    public void changeGeneralColor(Color newColor) {
        color = newColor;
    }

    /**
     * Gets the color assigned to an object.
     */
    public Color getColor() {
        return color;
    }

    /**
     * Gets the current color of an object.
     */
    public Color getCurrentColor() {
        return currentColor;
    }

    /**
     * Gets the default color of an object.
     */
    public Color getDefaultColor() {
        return defaultColor;
    }

    /**
     * Zeichne dieses Objekt mit seinen aktuellen Werten auf den Bildschirm.
     */
    private void draw() {
        if (isVisible) {
            Shape figur = getCurrentFigure();
            Canvas canvas = Canvas.getCanvas();
            canvas.drawFigure(this, // leinwand kennt das Objekt
                    currentColor, // defines the current color
                    figur); // definiert seinen grafischen Aspekt
            canvas.wait(10);
        }
    }

    /**
     * Loesche dieses Objekt vom Bildschirm.
     */
    private void delete() {
        if (isVisible) {
            Canvas leinwand = Canvas.getCanvas();
            leinwand.removeFigure(this);
        }
    }
}
